@@ -59,7 +59,7 @@ The new version of the list, minus the deleted strings, is returned."
 	     (skip-chars-forward "\t\f\n\r ")
 	     (setq start (point))
 	     (while (not (eobp))
-	       (skip-chars-forward "^\"\\,(")
+	       (skip-chars-forward "^\"\\\\,(")
 	       (setq char (following-char))
 	       (cond ((= char ?\\)
 		      (forward-char 1)
@@ -73,7 +73,7 @@ The new version of the list, minus the deleted strings, is returned."
 		      (skip-chars-forward ",\t\f\n\r ")
 		      (setq start (point)))
 		     ((= char ?\")
-		      (re-search-forward "[^\\]\"" nil 0))
+		      (re-search-forward "[^\\\\]\"" nil 0))
 		     ((= char ?\()
 		      (let ((parens 1))
 			(forward-char 1)
@@ -99,7 +99,7 @@ The new version of the list, minus the deleted strings, is returned."
       (save-excursion
        (unwind-protect
 	   (let ((list nil)
-		 (nonspecials "^\"\\( \t\n\r\f")
+		 (nonspecials "^\"\\\\( \t\n\r\f")
 		 start s char sp+sepchar)
 	     (if sepchar
 		 (setq nonspecials (concat nonspecials (list sepchar))
@@ -135,7 +135,7 @@ The new version of the list, minus the deleted strings, is returned."
 			    (forward-char 1)
 			  (delete-char 1))
 			(while (not done)
-			  (if (null (re-search-forward "[\\\"]" nil t))
+			  (if (null (re-search-forward "[\\\\\"]" nil t))
 			      (setq done t)
 			    (setq char (char-after (1- (point))))
 			    (cond ((char-equal char ?\\)
@@ -152,7 +152,7 @@ The new version of the list, minus the deleted strings, is returned."
 			    (parens 1))
 			(forward-char 1)
 			(while (not done)
-			  (if (null (re-search-forward "[\\()]" nil t))
+			  (if (null (re-search-forward "[\\\\()]" nil t))
 			      (setq done t)
 			    (setq char (char-after (1- (point))))
 			    (cond ((char-equal char ?\\)
@@ -172,6 +172,8 @@ The new version of the list, minus the deleted strings, is returned."
 	     (nreverse list))
 	(and work-buffer (kill-buffer work-buffer)))))))
 
+(defvar buffer-file-type)
+
 (defun vm-write-string (where string)
   (if (bufferp where)
       (vm-save-buffer-excursion
@@ -453,7 +455,7 @@ If HACK-ADDRESSES is t, then the strings are considered to be mail addresses,
   (cond (vm-xemacs-p
 	 (memq 'win (device-matching-specifier-tag-list)))
 	(vm-fsfemacs-p
-	 (eq window-system 'x))))
+	 (memq window-system '(x win32)))))
 
 (defun vm-menu-support-possible-p ()
   (cond (vm-xemacs-p
@@ -466,11 +468,14 @@ If HACK-ADDRESSES is t, then the strings are considered to be mail addresses,
 
 (defun vm-multiple-fonts-possible-p ()
   (cond (vm-xemacs-p
-	 (eq (device-type) 'x))
+	 (memq (device-type) '(x mswindows)))
 	(vm-fsfemacs-p
 	 (or (eq window-system 'x)
 	     (eq window-system 'win32)))))
 
+(defun vm-images-possible-here-p ()
+  (and vm-xemacs-p (memq (device-type) '(x mswindows))))
+
 (defun vm-run-message-hook (message &optional hook-variable)
   (save-excursion
     (set-buffer (vm-buffer-of message))
@@ -554,6 +559,7 @@ If HACK-ADDRESSES is t, then the strings are considered to be mail addresses,
 	((integerp vm-startup-with-summary)
 	 (let ((n vm-startup-with-summary))
 	   (cond ((< n 0) (null (nth (vm-abs n) vm-message-list)))
+		 ((= n 0) nil)
 		 (t (nth (1- n) vm-message-list)))))
 	(vm-startup-with-summary t)
 	(t nil)))
@@ -662,12 +668,14 @@ If HACK-ADDRESSES is t, then the strings are considered to be mail addresses,
       (vm-set-extent-property ee (car props) (car (cdr props)))
       (setq props (cdr (cdr props))))))
 
-(defun vm-make-tempfile-name ()
-  (let ((done nil) (pid (emacs-pid)) filename)
+(defun vm-make-tempfile-name (&optional filename-suffix)
+  (let ((done nil) filename)
     (while (not done)
-      (setq filename (expand-file-name (format "vm%d.%d" pid
-					       vm-tempfile-counter)
-				       vm-temp-file-directory)
+      (setq filename (convert-standard-filename
+		      (expand-file-name (format "vm%d%s"
+						vm-tempfile-counter
+					       (or filename-suffix ""))
+					vm-temp-file-directory))
 	    vm-tempfile-counter (1+ vm-tempfile-counter)
 	    done (not (file-exists-p filename))))
     filename ))
