@@ -1,5 +1,5 @@
 ;;; Simple POP (RFC 1939) client for VM
-;;; Copyright (C) 1993, 1994, 1997 Kyle E. Jones
+;;; Copyright (C) 1993, 1994, 1997, 1998 Kyle E. Jones
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -17,13 +17,26 @@
 
 (provide 'vm-pop)
 
-;; Nothing fancy here.
+(if (fboundp 'define-error)
+    (progn
+      (define-error 'vm-cant-uidl "Can't use UIDL")
+      (define-error 'vm-dele-failed "DELE command failed")
+      (define-error 'vm-uidl-failed "UIDL command failed"))
+  (put 'vm-cant-uidl 'error-conditions '(vm-cant-uidl error))
+  (put 'vm-cant-uidl 'error-message "Can't use UIDL")
+  (put 'vm-dele-failed 'error-conditions '(vm-dele-failed error))
+  (put 'vm-dele-failed 'error-message "DELE command failed")
+  (put 'vm-uidl-failed 'error-conditions '(vm-uidl-failed error))
+  (put 'vm-uidl-failed 'error-message "UIDL command failed"))
+
 ;; Our goal is to drag the mail from the POP maildrop to the crash box.
 ;; just as if we were using movemail on a spool file.
+;; We remember which messages we have retrieved so that we can
+;; leave the message in the mailbox, and yet not retrieve the
+;; same messages again and again.
 (defun vm-pop-move-mail (source destination)
   (let ((process nil)
 	(folder-type vm-folder-type)
-	(saved-password t)
 	(m-per-session vm-pop-messages-per-session)
 	(b-per-session vm-pop-bytes-per-session)
 	(handler (and (fboundp 'find-file-name-handler)
@@ -33,8 +46,18 @@
 			  (find-file-name-handler source)))))
 	(popdrop (vm-safe-popdrop-string source))
 	(statblob nil)
+	(can-uidl t)
+	(msgid (list nil (vm-popdrop-sans-password source) 'uidl))
+	(pop-retrieved-messages vm-pop-retrieved-messages)
+	auto-expunge x
 	mailbox-count mailbox-size message-size response
-	n retrieved retrieved-bytes process-buffer)
+	n retrieved retrieved-bytes process-buffer uidl)
+    (setq auto-expunge (cond ((setq x (assoc source vm-pop-auto-expunge-alist))
+			      (cdr x))
+			     ((setq x (assoc (vm-popdrop-sans-password source)
+					     vm-pop-auto-expunge-alist))
+			      (cdr x))
+			     (t vm-pop-expunge-after-retrieving)))
     (unwind-protect
 	(catch 'done
 	  (if handler
@@ -67,30 +90,57 @@
 			    (< retrieved m-per-session))
 			(or (not (natnump b-per-session))
 			    (< retrieved-bytes b-per-session)))
-	      (vm-set-pop-stat-x-currmsg statblob n)
-	      (vm-pop-send-command process (format "LIST %d" n))
-	      (setq message-size (vm-pop-read-list-response process))
-	      (vm-set-pop-stat-x-need statblob message-size)
-	      (if (and (integerp vm-pop-max-message-size)
-		       (> message-size vm-pop-max-message-size)
-		       (progn
-			 (setq response
-			       (if vm-pop-ok-to-ask
-				   (vm-pop-ask-about-large-message process
-								   message-size
-								   n)
-				 'skip))
-			 (not (eq response 'retrieve))))
-		  (if (eq response 'delete)
-		      (progn
-			(message "Deleting message %d..." n)
-			(vm-pop-send-command process (format "DELE %d" n))
-			(and (null (vm-pop-read-response process))
-			     (throw 'done (not (equal retrieved 0)))))
-		    (if vm-pop-ok-to-ask
-			(message "Skipping message %d..." n)
-	     (message "Skipping message %d in %s, too large (%d > %d)..."
-		      n popdrop message-size vm-pop-max-message-size)))
+	      (catch 'skip
+		(vm-set-pop-stat-x-currmsg statblob n)
+		(if can-uidl
+		    (condition-case nil
+			(let (list)
+			  (vm-pop-send-command process (format "UIDL %d" n))
+			  (setq response (vm-pop-read-response process t))
+			  (if (null response)
+			      (signal 'vm-cant-uidl nil))
+			  (setq list (vm-parse response "\\([\041-\176]+\\) *")
+				uidl (nth 2 list))
+			  (if (null uidl)
+			      (signal 'vm-cant-uidl nil))
+			  (setcar msgid uidl)
+			  (if (member msgid pop-retrieved-messages)
+			      (progn
+				(if vm-pop-ok-to-ask
+				    (message
+				     "Skipping message %d (of %d) from %s (retrieved already)..."
+				     n mailbox-count popdrop))
+				(throw 'skip t))))
+		      (vm-cant-uidl
+		       ;; something failed, so UIDL must not be working.
+		       ;; note that fact and carry on.
+		       (setq can-uidl nil
+			     msgid nil))))
+		(vm-pop-send-command process (format "LIST %d" n))
+		(setq message-size (vm-pop-read-list-response process))
+		(vm-set-pop-stat-x-need statblob message-size)
+		(if (and (integerp vm-pop-max-message-size)
+			 (> message-size vm-pop-max-message-size)
+			 (progn
+			   (setq response
+				 (if vm-pop-ok-to-ask
+				     (vm-pop-ask-about-large-message
+				      process popdrop message-size n)
+				   'skip))
+			   (not (eq response 'retrieve))))
+		    (progn
+		      (if (eq response 'delete)
+			  (progn
+			    (message "Deleting message %d..." n)
+			    (vm-pop-send-command process (format "DELE %d" n))
+			    (and (null (vm-pop-read-response process))
+				 (throw 'done (not (equal retrieved 0)))))
+			(if vm-pop-ok-to-ask
+			    (message "Skipping message %d..." n)
+			  (message
+			   "Skipping message %d in %s, too large (%d > %d)..."
+			   n popdrop message-size vm-pop-max-message-size)))
+		      (throw 'skip t)))
 		(message "Retrieving message %d (of %d) from %s..."
 			 n mailbox-count popdrop)
 		(vm-pop-send-command process (format "RETR %d" n))
@@ -102,14 +152,24 @@
 		(vm-increment retrieved)
 		(and b-per-session
 		     (setq retrieved-bytes (+ retrieved-bytes message-size)))
-		(vm-pop-send-command process (format "DELE %d" n))
-		;; DELE can't fail but Emacs or this code might
-		;; blow a gasket and spew filth down the
-		;; connection, so...
-		(and (null (vm-pop-read-response process))
-		     (throw 'done (not (equal retrieved 0)))))
+		(if (and (not auto-expunge) msgid)
+		    (setq pop-retrieved-messages
+			  (cons (copy-sequence msgid)
+				pop-retrieved-messages))
+		  ;; Either the user doesn't want the messages
+		  ;; kept in the mailbox or there's no UIDL
+		  ;; support so there's no way to remember what
+		  ;; messages we've retrieved.  Delete the
+		  ;; message now.
+		  (vm-pop-send-command process (format "DELE %d" n))
+		  ;; DELE can't fail but Emacs or this code might
+		  ;; blow a gasket and spew filth down the
+		  ;; connection, so...
+		  (and (null (vm-pop-read-response process))
+		       (throw 'done (not (equal retrieved 0))))))
 	      (vm-increment n))
 	     (not (equal retrieved 0)) ))
+      (setq vm-pop-retrieved-messages pop-retrieved-messages)
       (and statblob (vm-pop-stop-status-timer statblob))
       (if process
 	  (vm-pop-end-session process)))))
@@ -121,7 +181,9 @@
 			  (find-file-name-handler source 'vm-pop-check-mail)
 			(wrong-number-of-arguments
 			 (find-file-name-handler source)))))
-	response)
+	(retrieved vm-pop-retrieved-messages)
+	(popdrop (vm-popdrop-sans-password source))
+	x response)
     (unwind-protect
 	(save-excursion
 	  (catch 'done
@@ -131,6 +193,23 @@
 	    (setq process (vm-pop-make-session source))
 	    (or process (throw 'done nil))
 	    (set-buffer (process-buffer process))
+	    (vm-pop-send-command process "UIDL")
+	    (setq response (vm-pop-read-uidl-long-response process))
+	    (if (null response)
+		;; server doesn't understand UIDL
+		nil
+	      (if (null (car response))
+		  ;; (nil . nil) is returned if there are no
+		  ;; messages in the mailbox.
+		  (throw 'done nil)
+		(while response
+		  (if (not (and (setq x (assoc (cdr (car response)) retrieved))
+				(equal (nth 1 x) popdrop)
+				(eq (nth 2 x) 'uidl)))
+		      (throw 'done t))
+		  (setq response (cdr response))))
+	      ;; all messages in the mailbox have already been retrieved
+	      (throw 'done nil))
 	    (vm-pop-send-command process "STAT")
 	    (setq response (vm-pop-read-stat-response process))
 	    (if (null response)
@@ -138,13 +217,122 @@
 	      (not (equal 0 (car response))))))
       (and process (vm-pop-end-session process)))))
 
+(defun vm-expunge-pop-messages ()
+  "Deletes all messages from POP mailbox that have already been retrieved
+into the current folder.  VM sends POP DELE commands to all the
+relevant POP servers to remove the messages."
+  (interactive)
+  (vm-follow-summary-cursor)
+  (vm-select-folder-buffer)
+  (vm-check-for-killed-summary)
+  (vm-error-if-virtual-folder)
+  (let ((process nil)
+	(source nil)
+	(trouble nil)
+	(delete-count 0)
+	(vm-block-new-mail t)
+	(vm-pop-ok-to-ask t)
+	popdrop uidl-alist data mp match)
+    (unwind-protect
+	(save-excursion
+	  (setq vm-pop-retrieved-messages
+		(sort vm-pop-retrieved-messages
+		      (function (lambda (a b)
+				  (cond ((string-lessp (nth 1 a) (nth 1 b)) t)
+					((string-lessp (nth 1 b)
+						       (nth 1 a))
+					 nil)
+					((string-lessp (car a) (car b)) t)
+					(t nil))))))
+	  (setq mp vm-pop-retrieved-messages)
+	  (while mp
+	    (condition-case nil
+		(catch 'replay
+		  (setq data (car mp))
+		  (if (not (equal source (nth 1 data)))
+		      (progn
+			(if process
+			    (progn
+			     (vm-pop-end-session process)
+			     (setq process nil)))
+			(setq source (nth 1 data))
+			(setq popdrop (vm-safe-popdrop-string source))
+			(condition-case nil
+			    (progn
+			      (message "Opening POP session to %s..." popdrop)
+			      (setq process (vm-pop-make-session source))
+			      (if (null process)
+				  (signal 'error nil))
+			      (message "Expunging messages in %s..." popdrop))
+			  (error
+			   (message
+			    "Couldn't open POP session to %s, skipping..."
+			    popdrop)
+			   (setq trouble (cons popdrop trouble))
+			   (sleep-for 2)
+			   (while (equal (nth 1 (car mp)) source)
+			     (setq mp (cdr mp)))
+			   (throw 'replay t)))
+			(set-buffer (process-buffer process))
+			(vm-pop-send-command process "UIDL")
+			(setq uidl-alist
+			      (vm-pop-read-uidl-long-response process))
+			(if (null uidl-alist)
+			    (signal 'vm-uidl-failed nil))))
+		  (if (setq match (rassoc (car data) uidl-alist))
+		      (progn
+			(vm-pop-send-command process
+					     (format "DELE %s" (car match)))
+			(if (null (vm-pop-read-response process))
+			    (signal 'vm-dele-failed nil))
+			(vm-increment delete-count)))
+		  (setq mp (cdr mp)))
+	      (vm-dele-failed
+	       (message "DELE %s failed on %s, skipping rest of mailbox..."
+			(car match) popdrop)
+	       (setq trouble (cons popdrop trouble))
+	       (sleep-for 2)
+	       (while (equal (nth 1 (car mp)) source)
+		 (setq mp (cdr mp)))
+	       (throw 'replay t))
+	      (vm-uidl-failed
+	       (message "UIDL %s failed on %s, skipping this mailbox..."
+			(car match) popdrop)
+	       (setq trouble (cons popdrop trouble))
+	       (sleep-for 2)
+	       (while (equal (nth 1 (car mp)) source)
+		 (setq mp (cdr mp)))
+	       (throw 'replay t))))
+	  (if trouble
+	      (progn
+		(set-buffer (get-buffer-create "*POP Expunge Trouble*"))
+		(setq buffer-read-only nil)
+		(erase-buffer)
+		(insert (format "%s POP message%s expunged.\n\n"
+				(if (zerop delete-count) "No" delete-count)
+				(if (= delete-count 1) "" "s")))
+		(insert "VM had problems expunging messages from:\n")
+		(nreverse trouble)
+		(setq mp trouble)
+		(while mp
+		  (insert "   " (car mp) "\n")
+		  (setq mp (cdr mp)))
+		(setq buffer-read-only t)
+		(display-buffer (current-buffer)))
+	    (message "%s POP message%s expunged."
+		     (if (zerop delete-count) "No" delete-count)
+		     (if (= delete-count 1) "" "s"))))
+      (and process (vm-pop-end-session process)))
+    (or trouble (setq vm-pop-retrieved-messages nil))))
+
 (defun vm-pop-make-session (source)
   (let ((process-to-shutdown nil)
 	process
-	(saved-password t)
 	(popdrop (vm-safe-popdrop-string source))
+	(coding-system-for-read 'binary)
+	(coding-system-for-write 'binary)
 	greeting timestamp
-	host port auth user pass source-list process-buffer)
+	host port auth user pass source-list process-buffer source-nopwd)
     (unwind-protect
 	(catch 'done
 	  ;; parse the maildrop
@@ -153,7 +341,8 @@
 		port (nth 1 source-list)
 		auth (nth 2 source-list)
 		user (nth 3 source-list)
-		pass (nth 4 source-list))
+		pass (nth 4 source-list)
+		source-nopwd (vm-popdrop-sans-password source))
 	  ;; carp if parts are missing
 	  (if (null host)
 	      (error "No host in POP maildrop specification, \"%s\""
@@ -175,7 +364,7 @@
 		     source))
 	  (if (equal pass "*")
 	      (progn
-		(setq pass (car (cdr (assoc source vm-pop-passwords))))
+		(setq pass (car (cdr (assoc source-nopwd vm-pop-passwords))))
 		(if (null pass)
 		    (if (null vm-pop-ok-to-ask)
 			(progn (message "Need password for %s" popdrop)
@@ -183,19 +372,24 @@
 		      (setq pass
 			    (vm-read-password
 			     (format "POP password for %s: "
-				     popdrop))
-			    vm-pop-passwords (cons (list source pass)
-						   vm-pop-passwords)
-			    saved-password t)))))
+				     popdrop)))))))
+	  ;; save the password for the sake of
+	  ;; vm-expunge-pop-messages, which passes password-less
+	  ;; popdrop specifications to vm-make-pop-session.
+	  (if (null (assoc source-nopwd vm-pop-passwords))
+	      (setq vm-pop-passwords (cons (list source-nopwd pass)
+					   vm-pop-passwords)))
 	  ;; get the trace buffer
 	  (setq process-buffer
-		(get-buffer-create (format "trace of POP session to %s" host)))
+		(generate-new-buffer (format "trace of POP session to %s"
+					     host)))
 	  ;; Tell XEmacs/MULE not to mess with the text.
 	  (and vm-xemacs-mule-p
-	       (set-file-coding-system 'binary t))
+	       (set-buffer-file-coding-system 'binary t))
 	  ;; clear the trace buffer of old output
 	  (save-excursion
 	    (set-buffer process-buffer)
+	    (buffer-disable-undo process-buffer)
 	    (erase-buffer))
 	  ;; open the connection to the server
 	  (setq process (open-network-stream "POP" process-buffer host port))
@@ -217,10 +411,11 @@
 		   (vm-pop-send-command process (format "PASS %s" pass))
 		   (if (null (vm-pop-read-response process))
 		       (progn
-			 (if saved-password
-			     (setq vm-pop-passwords
-				   (delete (list source pass)
-					   vm-pop-passwords)))
+			 (setq vm-pop-passwords
+			       (delete (list source-nopwd pass)
+				       vm-pop-passwords))
+			 (message "POP password for %s incorrect" popdrop)
+			 (sleep-for 2)
 			 (throw 'done nil))))
 		  ((equal auth "rpop")
 		   (vm-pop-send-command process (format "USER %s" user))
@@ -254,7 +449,9 @@
   (save-excursion
     (set-buffer (process-buffer process))
     (vm-pop-send-command process "QUIT")
-    (vm-pop-read-response process)
+    ;; we don't care about the response
+    ;;(vm-pop-read-response process)
+    (kill-buffer (process-buffer process))
     (if (fboundp 'add-async-timeout)
 	(add-async-timeout 2 'delete-process process)
       (run-at-time 2 nil 'delete-process process))))
@@ -367,7 +564,45 @@
   (let ((response (vm-pop-read-response process t)))
     (string-to-int (nth 2 (vm-parse response "\\([^ ]+\\) *")))))
 
-(defun vm-pop-ask-about-large-message (process size n)
+(defun vm-pop-read-uidl-long-response (process)
+  (let ((start vm-pop-read-point)
+	(list nil)
+	n uidl)
+    (catch 'done
+      (goto-char start)
+      (while (not (re-search-forward "^\\.\r\n\\|^-ERR .*$" nil 0))
+	(beginning-of-line)
+	;; save-excursion doesn't work right
+	(let ((opoint (point)))
+	  (accept-process-output process)
+	  (goto-char opoint)))
+      (setq vm-pop-read-point (point-marker))
+      (goto-char start)
+      ;; no uidl support, bail.
+      (if (not (looking-at "\\+OK"))
+	  (throw 'done nil))
+      (forward-line 1)
+      (while (not (eq (char-after (point)) ?.))
+	;; not loking at a number, bail.
+	(if (not (looking-at "[0-9]"))
+	    (throw 'done nil))
+	(setq n (int-to-string (read (current-buffer))))
+	(skip-chars-forward " ")
+	(setq start (point))
+	(skip-chars-forward "\041-\176")
+	;; no tag after the message number, bail.
+	(if (= start (point))
+	    (throw 'done nil))
+	(setq uidl (buffer-substring start (point)))
+	(setq list (cons (cons n uidl) list))
+	(forward-line 1))
+      ;; returning nil means the uidl command failed so don't
+      ;; return nil if there aren't any messages.
+      (if (null list)
+	  (cons nil nil)
+	list ))))
+
+(defun vm-pop-ask-about-large-message (process popdrop size n)
   (let ((work-buffer nil)
 	(pop-buffer (current-buffer))
 	start end)
@@ -380,7 +615,9 @@
 		  (setq start vm-pop-read-point)
 		  (vm-pop-read-past-dot-sentinel-line process)
 		  (setq end vm-pop-read-point)
-		  (setq work-buffer (generate-new-buffer "*pop-glop*"))
+		  (setq work-buffer (generate-new-buffer
+				     (format "*headers of %s message %d*"
+					     popdrop n)))
 		  (set-buffer work-buffer)
 		  (insert-buffer-substring pop-buffer start end)
 		  (forward-line -1)
@@ -480,7 +717,7 @@
       (replace-match "" t t)
       (forward-char)))
   (if (> (- end start) 30000)
-      (message "CRLF conversion and dot unstuffing... done"))
+      (message "CRLF conversion and char unstuffing... done"))
   (set-marker end nil))
 
 (defun vm-pop-md5 (string)
@@ -498,3 +735,11 @@
 	  ;; viewing, make sure we leave it behind.
 	  (vm-buffer-substring-no-properties (point-min) (+ (point-min) 32)))
       (and buffer (kill-buffer buffer)))))
+
+(defun vm-popdrop-sans-password (source)
+  (let (source-list)
+    (setq source-list (vm-parse source "\\([^:]+\\):?"))
+    (concat (nth 0 source-list) ":"
+	    (nth 1 source-list) ":"
+	    (nth 2 source-list) ":"
+	    (nth 3 source-list) ":*")))
