@@ -42,7 +42,7 @@
 		in-reply-to
 		(and vm-in-reply-to-format
 		     (let ((vm-summary-uninteresting-senders nil))
-		       (vm-sprintf 'vm-in-reply-to-format (car mp))))
+		       (vm-summary-sprintf vm-in-reply-to-format (car mp))))
 		in-reply-to (and (not (equal "" in-reply-to)) in-reply-to))
 	  (and subject (stringp vm-reply-subject-prefix)
 	       (let ((case-fold-search t))
@@ -84,11 +84,10 @@
 		      (setq cc (concat cc "," tmp2))
 		    (setq cc tmp2)))))
 	(setq references
-	      (cons (vm-get-header-contents (car mp) "References:" " ")
-		    (cons (vm-get-header-contents (car mp) "In-reply-to:" " ")
-			  (cons (vm-get-header-contents (car mp) "Message-ID:"
-							" ")
-				references))))
+	      (cons (or (vm-get-header-contents (car mp) "References:" " ")
+			(vm-get-header-contents (car mp) "In-reply-to:" " "))
+		    (cons (vm-get-header-contents (car mp) "Message-ID:" " ")
+			  references)))
 	(setq newsgroups
 	      (cons (or (and to-all (vm-get-header-contents (car mp) "Followup-To:" ","))
 			(vm-get-header-contents (car mp) "Newsgroups:" ","))
@@ -177,7 +176,8 @@
 	(delete-char 1))
     (if vm-included-text-attribution-format
 	(let ((vm-summary-uninteresting-senders nil))
-	  (insert (vm-sprintf 'vm-included-text-attribution-format message))))
+	  (insert (vm-summary-sprintf vm-included-text-attribution-format
+				      message))))
     ; turn off zmacs-regions for Lucid Emacs 19
     ; and get around transient-mark-mode in FSF Emacs 19
     ; all this so that (mark) does what it did in v18, sheesh.
@@ -409,7 +409,14 @@ as having been replied to, if appropriate."
     (save-restriction
       (save-excursion
 	(let ((resent nil)
+	      hostname
 	      (time (current-time)))
+	  (setq hostname (cond ((string-match "\\." (system-name))
+				(system-name))
+			       ((and (stringp 'mail-host-address)
+				     (string-match "\\." mail-host-address))
+				mail-host-address)
+			       (t "gargle.gargle.HOWL")))
 	  (if (or (vm-mail-mode-get-header-contents "Resent-To:")
 		  (vm-mail-mode-get-header-contents "Resent-Cc:")
 		  (vm-mail-mode-get-header-contents "Resent-Bcc:"))
@@ -583,7 +590,7 @@ as replied to, forwarded, etc, if appropriate."
     (let ((case-fold-search nil))
       (if (not (string-match "^sent " (buffer-name)))
 	  (let (prefix name n)
-	    (if (not (= ?* (aref (buffer-name) 0)))
+	    (if (not (string-match "^mail to \\?" (buffer-name)))
 		(setq prefix (format "sent %s" (buffer-name)))
 	      (let (recipients)
 		(cond ((not (zerop (length (setq recipients
@@ -744,7 +751,8 @@ Subject: header manually."
 	 nil
 	 (and vm-forwarding-subject-format
 	      (let ((vm-summary-uninteresting-senders nil))
-		(vm-sprintf 'vm-forwarding-subject-format (car mp)))))
+		(vm-summary-sprintf vm-forwarding-subject-format
+				    (car mp)))))
 	(make-local-variable 'vm-forward-list)
 	(setq vm-system-state 'forwarding
 	      vm-forward-list (list (car mp))
@@ -772,7 +780,8 @@ Subject: header manually."
 		       (vm-determine-proper-content-transfer-encoding
 			(point)
 			(point-max))
-		       "\n"))
+		       "\n")
+	       (insert "Content-Description: forwarded message\n"))
 	      ((equal vm-forwarding-digest-type "rfc934")
 	       (vm-rfc934-encapsulate-messages
 		vm-forward-list vm-forwarded-headers
@@ -789,7 +798,8 @@ Subject: header manually."
 	  (let ((b (current-buffer)))
 	    (set-buffer mail-buffer)
 	    (mail-text)
-	    (vm-mime-attach-object b "message/rfc822" nil nil t)
+	    (vm-mime-attach-object b "message/rfc822" nil
+				   "forwarded message" t)
 	    (add-hook 'kill-buffer-hook
 		      (list 'lambda ()
 			    (list 'if (list 'eq mail-buffer '(current-buffer))
@@ -898,7 +908,14 @@ You may also create a Resent-Cc header."
 	  (insert "Resent-From: " vm-mail-header-from ?\n))
       (insert "Resent-To: \n")
       (if mail-self-blind
-	  (insert "Bcc: " (user-login-name) ?\n))
+	  (insert "Bcc: "
+		  (cond ((and vm-xemacs-p (fboundp 'user-mail-address))
+			 (user-mail-address))
+			((and (boundp 'user-mail-address)
+			      (stringp user-mail-address))
+			 user-mail-address)
+			(t (user-login-name)))
+		  ?\n))
       (if mail-archive-file-name
 	  (insert "FCC: " mail-archive-file-name ?\n))
       ;; delete all but pertinent headers
@@ -1010,7 +1027,8 @@ only marked messages will be put into the digest."
 		  (mail-text)))
 	    (while mp
 	      (let ((vm-summary-uninteresting-senders nil))
-		(insert (vm-sprintf 'vm-digest-preamble-format (car mp)) "\n"))
+		(insert (vm-summary-sprintf vm-digest-preamble-format
+					    (car mp)) "\n"))
 	      (if vm-digest-center-preamble
 		  (progn
 		    (forward-char -1)
@@ -1089,18 +1107,23 @@ found, the current buffer remains selected."
 (defvar mail-aliases)
 (defvar mail-default-reply-to)
 (defvar mail-signature-file)
+(defvar mail-personal-alias-file)
 
 (defun vm-mail-internal
     (&optional buffer-name to subject in-reply-to cc references newsgroups)
   (let ((folder-buffer nil))
     (if (memq major-mode '(vm-mode vm-virtual-mode))
 	(setq folder-buffer (current-buffer)))
-    (set-buffer (generate-new-buffer (or buffer-name "*VM-mail*")))
+    (set-buffer (generate-new-buffer (or buffer-name "mail to ?")))
     ;; avoid trying to write auto-save files in potentially
     ;; unwritable directories.
     (setq default-directory (or vm-folder-directory (expand-file-name "~/")))
     (auto-save-mode (if auto-save-default 1 -1))
     (mail-mode)
+    ;; TM infests mail mode, uninfest it if VM's MIME stuff is in
+    ;; use.
+    (if vm-send-using-mime
+	(vm-mail-mode-remove-tm-hooks))
     (use-local-map vm-mail-mode-map)
     ;; make mail-mode-map the parent of this vm-mail-mode-map, if we can.
     ;; do it only once.
@@ -1143,7 +1166,14 @@ found, the current buffer remains selected."
     (if mail-default-reply-to
 	(insert "Reply-To: " mail-default-reply-to "\n"))
     (if mail-self-blind
-	(insert "Bcc: " (user-login-name) "\n"))
+	(insert "Bcc: "
+		(cond ((and vm-xemacs-p (fboundp 'user-mail-address))
+		       (user-mail-address))
+		      ((and (boundp 'user-mail-address)
+			    (stringp user-mail-address))
+		       user-mail-address)
+		      (t (user-login-name)))
+		?\n))
     (if mail-archive-file-name
 	(insert "FCC: " mail-archive-file-name "\n"))
     (if mail-default-headers
@@ -1374,9 +1404,9 @@ message."
 	       (vm-mime-encode-composition))
 	  (vm-remove-mail-mode-header-separator)
 	  (goto-char (point-min))
-	  (insert (vm-leading-message-separator 'From_))
+	  (insert (vm-leading-message-separator vm-default-From_-folder-type))
 	  (goto-char (point-max))
-	  (insert (vm-trailing-message-separator 'From_))
+	  (insert (vm-trailing-message-separator vm-default-From_-folder-type))
 	  (set-buffer-modified-p nil)
 	  ;; point of no return, don't kill it if the user quits
 	  (setq temp-buffer nil)
@@ -1394,6 +1424,13 @@ message."
 		      (list this-command) '(vm-mode startup)))
       (and temp-buffer (kill-buffer temp-buffer)))))
 
+(defun vm-mail-mode-remove-tm-hooks ()
+  (make-local-hook 'mail-setup-hook)
+  (remove-hook 'mail-setup-hook 'mime/decode-message-header t)
+  (remove-hook 'mail-setup-hook 'mime/editor-mode t)
+  (make-local-hook 'mail-send-hook)
+  (remove-hook 'mail-send-hook 'mime-editor/maybe-translate))
+
 (defvar mail-send-actions)
 
 (defun vm-compose-mail (&optional to subject other-headers continue
