@@ -293,7 +293,7 @@ See the documentation for vm-mode for more information."
 (defun vm-mode (&optional read-only)
   "Major mode for reading mail.
 
-This is VM 6.47.
+This is VM 6.62.
 
 Commands:
    h - summarize folder contents
@@ -306,6 +306,7 @@ Commands:
  M-n - go to next unread message
  M-p - go to previous unread message
  RET - go to numbered message (uses prefix arg or prompts in minibuffer)
+ M-g - go to numbered message (uses prefix arg or prompts in minibuffer)
  TAB - go to last message seen
    ^ - go to parent of this message
  M-s - incremental search through the folder
@@ -425,6 +426,7 @@ Variables:
    vm-arrived-messages-hook
    vm-auto-center-summary
    vm-auto-decode-mime-messages
+   vm-auto-displayed-mime-content-type-exceptions
    vm-auto-displayed-mime-content-types
    vm-auto-folder-alist
    vm-auto-folder-case-fold-search
@@ -439,6 +441,7 @@ Variables:
    vm-convert-folder-types
    vm-crash-box
    vm-crash-box-suffix
+   vm-default-From_-folder-type
    vm-default-folder-type
    vm-delete-after-archiving
    vm-delete-after-bursting
@@ -469,12 +472,18 @@ Variables:
    vm-highlighted-header-regexp
    vm-honor-page-delimiters
    vm-image-directory
+   vm-imap-auto-expunge-alist
+   vm-imap-bytes-per-session
+   vm-imap-expunge-after-retrieving
+   vm-imap-max-message-size
+   vm-imap-messages-per-session
    vm-index-file-suffix
    vm-in-reply-to-format
    vm-included-text-attribution-format
    vm-included-text-discard-header-regexp
    vm-included-text-headers
    vm-included-text-prefix
+   vm-infer-mime-types
    vm-invisible-header-regexp
    vm-jump-to-new-messages
    vm-jump-to-unread-messages
@@ -507,8 +516,10 @@ Variables:
    vm-mime-display-function
    vm-mime-external-content-types-alist
    vm-mime-ignore-mime-version
+   vm-mime-internal-content-type-exceptions
    vm-mime-internal-content-types
    vm-mime-max-message-size
+   vm-mime-type-converter-alist
    vm-mode-hook
    vm-mosaic-program
    vm-mosaic-program-switches
@@ -532,6 +543,7 @@ Variables:
    vm-preview-read-messages
    vm-primary-inbox
    vm-quit-hook
+   vm-recognize-imap-maildrops
    vm-recognize-pop-maildrops
    vm-reply-hook
    vm-reply-ignored-addresses
@@ -606,6 +618,7 @@ visited folder."
   (interactive
    (save-excursion
      (vm-session-initialization)
+     (vm-check-for-killed-folder)
      (vm-select-folder-buffer)
      (let ((default-directory (if vm-folder-directory
 				  (expand-file-name vm-folder-directory)
@@ -622,6 +635,7 @@ visited folder."
 	      default-directory default nil nil 'vm-folder-history)
 	     current-prefix-arg))))
   (vm-session-initialization)
+  (vm-check-for-killed-folder)
   (vm-select-folder-buffer)
   (vm-check-for-killed-summary)
   (setq vm-last-visit-folder folder)
@@ -635,6 +649,7 @@ visited folder."
   (interactive
    (save-excursion
      (vm-session-initialization)
+     (vm-check-for-killed-folder)
      (vm-select-folder-buffer)
      (let ((default-directory (if vm-folder-directory
 				  (expand-file-name vm-folder-directory)
@@ -650,6 +665,7 @@ visited folder."
 			""))
 	      default-directory default nil nil 'vm-folder-history)
 	     current-prefix-arg))))
+  (vm-session-initialization)
   (if (vm-multiple-frames-possible-p)
       (vm-goto-new-frame 'folder))
   (let ((vm-frame-per-folder nil)
@@ -664,6 +680,7 @@ visited folder."
   (interactive
    (save-excursion
      (vm-session-initialization)
+     (vm-check-for-killed-folder)
      (vm-select-folder-buffer)
      (let ((default-directory (if vm-folder-directory
 				  (expand-file-name vm-folder-directory)
@@ -848,6 +865,7 @@ Optional argument TO is a string that should contain a comma separated
 recipient list."
   (interactive)
   (vm-session-initialization)
+  (vm-check-for-killed-folder)
   (vm-select-folder-buffer)
   (vm-check-for-killed-summary)
   (vm-mail-internal nil to)
@@ -888,6 +906,8 @@ recipient list."
   "Submit a bug report, with pertinent information to the VM bug list."
   (interactive)
   (require 'reporter)
+  (require 'vm-version)
+  (require 'vm-vars)
   ;; make sure the user doesn't try to use vm-mail here.
   (let ((reporter-mailer '(mail)))
     (delete-other-windows)
@@ -899,6 +919,7 @@ recipient list."
       'vm-arrived-messages-hook
       'vm-auto-center-summary
       'vm-auto-decode-mime-messages
+      'vm-auto-displayed-mime-content-type-exceptions
       'vm-auto-displayed-mime-content-types
 ;; don't send this by default, might be personal stuff in here.
 ;;      'vm-auto-folder-alist
@@ -913,6 +934,7 @@ recipient list."
       'vm-convert-folder-types
       'vm-crash-box
       'vm-crash-box-suffix
+      'vm-default-From_-folder-type
       'vm-default-folder-type
       'vm-delete-after-archiving
       'vm-delete-after-bursting
@@ -946,6 +968,12 @@ recipient list."
       'vm-highlighted-header-regexp
       'vm-honor-page-delimiters
       'vm-image-directory
+;; IMAP passwords might be listed here
+;;      'vm-imap-auto-expunge-alist
+      'vm-imap-bytes-per-session
+      'vm-imap-expunge-after-retrieving
+      'vm-imap-max-message-size
+      'vm-imap-messages-per-session
       'vm-in-reply-to-format
       'vm-included-text-attribution-format
       'vm-included-text-discard-header-regexp
@@ -953,6 +981,7 @@ recipient list."
       'vm-included-text-prefix
       'vm-index-file-suffix
       'vm-init-file
+      'vm-infer-mime-types
       'vm-invisible-header-regexp
       'vm-jump-to-new-messages
       'vm-jump-to-unread-messages
@@ -986,8 +1015,10 @@ recipient list."
       'vm-mime-display-function
       'vm-mime-external-content-types-alist
       'vm-mime-ignore-mime-version
+      'vm-mime-internal-content-type-exceptions
       'vm-mime-internal-content-types
       'vm-mime-max-message-size
+      'vm-mime-type-converter-alist
       'vm-mode-hook
       'vm-mode-hooks
       'vm-mosaic-program
@@ -1014,6 +1045,7 @@ recipient list."
       'vm-preview-read-messages
       'vm-primary-inbox
       'vm-quit-hook
+      'vm-recognize-imap-maildrops
       'vm-recognize-pop-maildrops
       'vm-reply-hook
 ;; don't feed the spammers or crackers
@@ -1087,7 +1119,10 @@ recipient list."
 (defun vm-load-init-file (&optional interactive)
   (interactive "p")
   (if (or (not vm-init-file-loaded) interactive)
-      (load vm-init-file (not interactive) (not interactive) t))
+      (progn
+	(and vm-init-file
+	     (load vm-init-file (not interactive) (not interactive) t))
+	(and vm-preferences-file (load vm-preferences-file t t t))))
   (setq vm-init-file-loaded t)
   (vm-display nil nil '(vm-load-init-file) '(vm-load-init-file)))
 
@@ -1139,8 +1174,19 @@ recipient list."
 	(setq vm-buffers-needing-display-update (make-vector 29 0))
 	;; default value of vm-mime-button-face is 'gui-button-face
 	;; this face doesn't exist by default in FSF Emacs 19.34.
-	;; Create it.
-	(and (fboundp 'make-face) (make-face 'gui-button-face))
+	;; Create it and initialize it to something reasonable.
+	(if (and vm-fsfemacs-p (featurep 'faces)
+		 (not (facep 'gui-button-face)))
+	    (progn
+	      (make-face 'gui-button-face)
+	      (cond ((eq window-system 'x)
+		     (set-face-foreground 'gui-button-face "black")
+		     (set-face-background 'gui-button-face "gray75"))
+		    (t
+		     ;; use primary color names, since fancier
+		     ;; names may not be valid.
+		     (set-face-foreground 'gui-button-face "white")
+		     (set-face-background 'gui-button-face "red")))))
 	(and (vm-mouse-support-possible-p)
 	     (vm-mouse-install-mouse))
 	(and (vm-menu-support-possible-p)
