@@ -32,17 +32,17 @@
 	truncate-lines t)
   ;; horizontal scrollbar off by default
   ;; user can turn it on in summary hook if desired.
-  (and (fboundp 'set-specifier)
-       scrollbar-height
+  (and vm-xemacs-p (featurep 'scrollbar)
        (set-specifier scrollbar-height (cons (current-buffer) 0)))
   (use-local-map vm-summary-mode-map)
   (and (vm-menu-support-possible-p)
        (vm-menu-install-menus))
-  (and vm-mouse-track-summary
-       (vm-mouse-support-possible-p)
-       (vm-mouse-xemacs-mouse-p)
-       (add-hook 'mode-motion-hook 'mode-motion-highlight-line))
-  (if (or vm-frame-per-folder vm-frame-per-summary)
+;; using the 'mouse-face property gives faster highlighting than this.
+;;  (and vm-mouse-track-summary
+;;       (vm-mouse-support-possible-p)
+;;       (vm-mouse-xemacs-mouse-p)
+;;       (add-hook 'mode-motion-hook 'mode-motion-highlight-line))
+  (if (and vm-mutable-frames (or vm-frame-per-folder vm-frame-per-summary))
       (vm-set-hooks-for-frame-deletion))
   (run-hooks 'vm-summary-mode-hook)
   ;; Lucid Emacs apparently used this name
@@ -105,15 +105,13 @@ mandatory."
 
 (defun vm-do-summary (&optional start-point)
   (let ((m-list (or start-point vm-message-list))
-	mp
+	mp m
 	(n 0)
 	;; Just for laughs, make the update interval vary.
 	(modulus (+ (% (vm-abs (random)) 11) 10))
-	(mouse-track-func
+	(do-mouse-track
 	    (and vm-mouse-track-summary
-		 (vm-mouse-support-possible-p)
-		 (vm-mouse-fsfemacs-mouse-p)
-		 (function vm-mouse-set-mouse-track-highlight)))
+		 (vm-mouse-support-possible-p)))
 	summary)
     (setq mp m-list)
     (save-excursion
@@ -148,11 +146,15 @@ mandatory."
 		  (message "Generating summary markers... "))
 	      (setq mp m-list)
 	      (while mp
-		(and mouse-track-func (funcall mouse-track-func
-					       (vm-su-start-of (car mp))
-					       (vm-su-end-of (car mp))))
-		(vm-set-su-start-of (car mp) (vm-marker (vm-su-start-of (car mp))))
-		(vm-set-su-end-of (car mp) (vm-marker (vm-su-end-of (car mp))))
+		(setq m (car mp))
+		(and do-mouse-track
+		     (vm-set-su-summary-mouse-track-overlay-of
+		      m
+		      (vm-mouse-set-mouse-track-highlight
+		       (vm-su-start-of m)
+		       (vm-su-end-of m))))
+		(vm-set-su-start-of m (vm-marker (vm-su-start-of m)))
+		(vm-set-su-end-of m (vm-marker (vm-su-end-of m)))
 		(setq mp (cdr mp))))
 	  (set-buffer-modified-p modified))
 	(run-hooks 'vm-summary-redo-hook)))
@@ -180,11 +182,9 @@ mandatory."
   (if (and (vm-su-start-of m)
 	   (marker-buffer (vm-su-start-of m)))
       (let ((modified (buffer-modified-p))
-	    (mouse-track-func
+	    (do-mouse-track
 	     (and vm-mouse-track-summary
-		  (vm-mouse-support-possible-p)
-		  (vm-mouse-fsfemacs-mouse-p)
-		  (function vm-mouse-set-mouse-track-highlight)))
+		  (vm-mouse-support-possible-p)))
 	    summary)
 	(save-excursion
 	  (setq summary (vm-su-summary m))
@@ -226,9 +226,11 @@ mandatory."
 		  (vm-tokenized-summary-insert m (vm-su-summary m))
 		  (delete-char 1)
 		  (run-hooks 'vm-summary-update-hook)
-		  (and mouse-track-func (funcall mouse-track-func
-						 (vm-su-start-of m)
-						 (vm-su-end-of m)))
+		  (and do-mouse-track
+		       (vm-mouse-set-mouse-track-highlight
+			(vm-su-start-of m)
+			(vm-su-end-of m)
+			(vm-su-summary-mouse-track-overlay-of m)))
 		  (if (and selected vm-summary-highlight-face)
 		      (vm-summary-highlight-region (vm-su-start-of m) (point)
 						   vm-summary-highlight-face)))
@@ -237,11 +239,9 @@ mandatory."
 (defun vm-set-summary-pointer (m)
   (if vm-summary-buffer
       (let ((w (vm-get-visible-buffer-window vm-summary-buffer))
-	    (mouse-track-func
+	    (do-mouse-track
 	       (and vm-mouse-track-summary
-		    (vm-mouse-support-possible-p)
-		    (vm-mouse-fsfemacs-mouse-p)
-		    (function vm-mouse-set-mouse-track-highlight)))
+		    (vm-mouse-support-possible-p)))
 	    (old-window nil))
 	(vm-save-buffer-excursion
 	  (unwind-protect
@@ -258,10 +258,12 @@ mandatory."
 			(goto-char (vm-su-start-of vm-summary-pointer))
 			(insert vm-summary-no-=>)
 			(delete-char (length vm-summary-=>))
-			(and mouse-track-func
-			     (funcall mouse-track-func
-				      (- (point) (length vm-summary-=>))
-					 (point)))))
+			(and do-mouse-track
+			     (vm-mouse-set-mouse-track-highlight
+			      (vm-su-start-of vm-summary-pointer)
+			      (vm-su-end-of vm-summary-pointer)
+			      (vm-su-summary-mouse-track-overlay-of
+			       vm-summary-pointer)))))
 		  (setq vm-summary-pointer m)
 		  (goto-char (vm-su-start-of m))
 		  (let ((modified (buffer-modified-p)))
@@ -269,10 +271,10 @@ mandatory."
 			(progn
 			  (insert vm-summary-=>)
 			  (delete-char (length vm-summary-=>))
-			  (and mouse-track-func
-			       (funcall mouse-track-func
-					(- (point) (length vm-summary-=>))
-					   (point))))
+			  (and do-mouse-track
+			       (vm-mouse-set-mouse-track-highlight
+				(vm-su-start-of m) (vm-su-end-of m)
+				(vm-su-summary-mouse-track-overlay-of m))))
 		      (set-buffer-modified-p modified)))
 		  (forward-char (- (length vm-summary-=>)))
 		  (if vm-summary-highlight-face
@@ -284,7 +286,7 @@ mandatory."
 	    (and old-window (select-window old-window)))))))
 
 (defun vm-summary-highlight-region (start end face)
-  (cond (vm-fsfemacs-19-p
+  (cond (vm-fsfemacs-p
 	 (if (and vm-summary-overlay (overlay-buffer vm-summary-overlay))
 	     (move-overlay vm-summary-overlay start end)
 	   (setq vm-summary-overlay (make-overlay start end))
@@ -309,16 +311,31 @@ mandatory."
       (if (or (eq vm-auto-center-summary t) (not (one-window-p t)))
 	  (recenter '(4)))))
 
-(defun vm-sprintf (format-variable message &optional tokenize)
+(defun vm-summary-sprintf (format message &optional tokenize)
   ;; compile the format into an eval'able s-expression
   ;; if it hasn't been compiled already.
-  (if (not (eq (get format-variable 'vm-compiled-format)
-	       (symbol-value format-variable)))
-      (vm-compile-format format-variable tokenize))
-  ;; The local variable name `vm-su-message' is mandatory here for
-  ;; the format s-expression to work.
-  (let ((vm-su-message message))
-    (eval (get format-variable 'vm-format-sexp))))
+  (let* ((alist-var (if tokenize
+			'vm-summary-tokenized-compiled-format-alist
+		      'vm-summary-untokenized-compiled-format-alist))
+	 (match (assoc format (symbol-value alist-var))))
+    (if (null match)
+	(progn
+	  (vm-summary-compile-format format tokenize)
+	  (setq match (assoc format (symbol-value alist-var)))))
+    ;; The local variable name `vm-su-message' is mandatory here for
+    ;; the format s-expression to work.
+    (let ((vm-su-message message))
+      (eval (cdr match)))))
+
+(defun vm-summary-compile-format (format tokenize)
+  (let ((return-value (vm-summary-compile-format-1 format tokenize)))
+    (if tokenize
+	(setq vm-summary-tokenized-compiled-format-alist
+	      (cons (cons format return-value)
+		    vm-summary-tokenized-compiled-format-alist))
+      (setq vm-summary-untokenized-compiled-format-alist
+	    (cons (cons format return-value)
+		  vm-summary-untokenized-compiled-format-alist)))))
 
 (defun vm-tokenized-summary-insert (message tokens)
   (if (stringp tokens)
@@ -341,9 +358,8 @@ mandatory."
 				      (vm-th-thread-indentation message))))))
 	(setq tokens (cdr tokens))))))
 
-(defun vm-compile-format (format-variable &optional tokenize)
-  (let ((format (symbol-value format-variable))
-	(case-fold-search nil)
+(defun vm-summary-compile-format-1 (format &optional tokenize)
+  (let ((case-fold-search nil)
 	(done nil)
 	(list nil)
 	(sexp nil)
@@ -450,20 +466,28 @@ mandatory."
 		     (setcar sexp
 			     (list 'vm-decode-mime-encoded-words-in-string
 				   (car sexp)))))
-	      (cond ((and (not token) (match-beginning 1))
+	      (cond ((and (not token) (match-beginning 1) (match-beginning 2))
 		     (setcar sexp
-			     (list 'vm-left-justify-string (car sexp)
-				   (string-to-int
-				    (substring format
-					       (match-beginning 2)
-					       (match-end 2))))))
+			     (list
+			      (if (eq (aref format (match-beginning 2)) ?0)
+				  'vm-numeric-left-justify-string
+				'vm-left-justify-string)
+			      (car sexp)
+			      (string-to-int
+			       (substring format
+					  (match-beginning 2)
+					  (match-end 2))))))
 		    ((and (not token) (match-beginning 2))
 		     (setcar sexp
-			     (list 'vm-right-justify-string (car sexp)
-				   (string-to-int
-				    (substring format
-					       (match-beginning 2)
-					       (match-end 2)))))))
+			     (list
+			      (if (eq (aref format (match-beginning 2)) ?0)
+				  'vm-numeric-right-justify-string
+				'vm-right-justify-string)
+			      (car sexp)
+			      (string-to-int
+			       (substring format
+					  (match-beginning 2)
+					  (match-end 2)))))))
 	      (cond ((and (not token) (match-beginning 3))
 		     (setcar sexp
 			     (list 'vm-truncate-string (car sexp)
@@ -502,8 +526,7 @@ mandatory."
 			    (and token (list token)))
 		sexp nil
 		sexp-fmt nil)))
-    (put format-variable 'vm-compiled-format format)
-    (put format-variable 'vm-format-sexp (if list (cons 'list list) sexp))))
+    (if list (cons 'list list) sexp)))
 
 (defun vm-get-header-contents (message header-name-regexp &optional clump-sep)
   (let ((contents nil)
@@ -536,6 +559,16 @@ mandatory."
       string
     (concat (make-string (- width (length string)) ?\ ) string)))
 
+(defun vm-numeric-left-justify-string (string width)
+  (if (>= (length string) width)
+      string
+    (concat string (make-string (- width (length string)) ?0))))
+
+(defun vm-numeric-right-justify-string (string width)
+  (if (>= (length string) width)
+      string
+    (concat (make-string (- width (length string)) ?0) string)))
+
 (defun vm-truncate-string (string width)
   (cond
 ;; doesn't work because the width of wide chars such as the Kanji
@@ -634,7 +667,7 @@ mandatory."
 (defun vm-grok-From_-date (message)
   ;; This works only on the From_ types, obviously
   (if (not (memq (vm-message-type-of message)
-		 '(From_ From_-with-Content-Length)))
+		 '(BellFrom_ From_ From_-with-Content-Length)))
       nil
     (save-excursion
       (set-buffer (vm-buffer-of (vm-real-message-of message)))
@@ -803,7 +836,7 @@ mandatory."
 (defun vm-grok-From_-author (message)
   ;; This works only on the From_ types, obviously
   (if (not (memq (vm-message-type-of message)
-		 '(From_ From_-with-Content-Length)))
+		 '(From_ BellFrom_ From_-with-Content-Length)))
       nil
     (save-excursion
       (set-buffer (vm-buffer-of message))
@@ -975,12 +1008,13 @@ mandatory."
       (or (vm-virtual-summary-of m)
 	  (save-excursion
 	    (vm-select-folder-buffer)
-	    (vm-set-virtual-summary-of m (vm-sprintf 'vm-summary-format m t))
+	    (vm-set-virtual-summary-of m (vm-summary-sprintf
+					  vm-summary-format m t))
 	    (vm-virtual-summary-of m)))
     (or (vm-summary-of m)
 	(save-excursion
 	  (vm-select-folder-buffer)
-	  (vm-set-summary-of m (vm-sprintf 'vm-summary-format m t))
+	  (vm-set-summary-of m (vm-summary-sprintf vm-summary-format m t))
 	  (vm-summary-of m)))))
 
 (defun vm-fix-my-summary!!! ()
